@@ -28,6 +28,7 @@ import stringcase
 from requests import Session
 
 from .casing import safe_snake_case
+from .utils import Response
 
 session = Session()
 
@@ -39,7 +40,7 @@ if TYPE_CHECKING:
 if not (sys.version_info.major == 3 and sys.version_info.minor >= 7):
     # Apply backport of datetime.fromisoformat from 3.7
     from backports.datetime_fromisoformat import MonkeyPatch
-
+    
     MonkeyPatch.patch_fromisoformat()
 
 # Proto 3 data types
@@ -122,7 +123,7 @@ DATETIME_ZERO = datetime(1970, 1, 1, tzinfo=timezone.utc)
 
 class Casing(enum.Enum):
     """Casing constants for serialization."""
-
+    
     CAMEL = stringcase.camelcase
     SNAKE = stringcase.snakecase
 
@@ -137,7 +138,7 @@ PLACEHOLDER: Any = _PLACEHOLDER()
 @dataclasses.dataclass(frozen=True)
 class FieldMetadata:
     """Stores internal metadata used for parsing & serialization."""
-
+    
     # Protobuf field number
     number: int
     # Protobuf type name
@@ -148,7 +149,7 @@ class FieldMetadata:
     group: Optional[str] = None
     # Describes the wrapped type (e.g. when using google.protobuf.BoolValue)
     wraps: Optional[str] = None
-
+    
     @staticmethod
     def get(field: dataclasses.Field) -> "FieldMetadata":
         """Returns the field metadata for a dataclass field."""
@@ -257,7 +258,7 @@ def map_field(
 
 class Enum(int, enum.Enum):
     """Protocol buffers enumeration base class. Acts like `enum.IntEnum`."""
-
+    
     @classmethod
     def from_string(cls, name: str) -> int:
         """Return the value which corresponds to the string name."""
@@ -282,10 +283,10 @@ def _pack_fmt(proto_type: str) -> str:
 def encode_varint(value: int) -> bytes:
     """Encodes a single varint value for serialization."""
     b: List[int] = []
-
+    
     if value < 0:
         value += 1 << 64
-
+    
     bits = value & 0x7F
     value >>= 7
     while value:
@@ -333,9 +334,9 @@ def _preprocess_single(proto_type: str, wraps: str, value: Any) -> bytes:
             if value is None:
                 return b""
             value = _get_wrapper(wraps)(value=value)
-
+        
         return bytes(value)
-
+    
     return value
 
 
@@ -349,7 +350,7 @@ def _serialize_single(
 ) -> bytes:
     """Serializes a single field and value."""
     value = _preprocess_single(proto_type, wraps, value)
-
+    
     output = b""
     if proto_type in WIRE_VARINT_TYPES:
         key = encode_varint(field_number << 3)
@@ -366,7 +367,7 @@ def _serialize_single(
             output += key + encode_varint(len(value)) + value
     else:
         raise NotImplementedError(proto_type)
-
+    
     return output
 
 
@@ -403,7 +404,7 @@ def parse_fields(value: bytes) -> Generator[ParsedField, None, None]:
         num_wire, i = decode_varint(value, i)
         number = num_wire >> 3
         wire_type = num_wire & 0x7
-
+        
         decoded: Any = None
         if wire_type == 0:
             decoded, i = decode_varint(value, i)
@@ -415,7 +416,7 @@ def parse_fields(value: bytes) -> Generator[ParsedField, None, None]:
             i += length
         elif wire_type == 5:
             decoded, i = value[i: i + 4], i + 4
-
+        
         yield ParsedField(
             number=number, wire_type=wire_type, value=decoded, raw=value[start:i]
         )
@@ -431,51 +432,51 @@ class Message(ABC):
     register the message fields which get used by the serializers and parsers
     to go between Python, binary and JSON protobuf message representations.
     """
-
+    
     _serialized_on_wire: bool
     _unknown_fields: bytes
     _group_map: Dict[str, dict]
-
+    
     def __post_init__(self) -> None:
         # Keep track of whether every field was default
         all_sentinel = True
-
+        
         # Set a default value for each field in the class after `__init__` has
         # already been run.
         group_map: Dict[str, dict] = {"fields": {}, "groups": {}}
         for field in dataclasses.fields(self):
             meta = FieldMetadata.get(field)
-
+            
             if meta.group:
                 # This is part of a one-of group.
                 group_map["fields"][field.name] = meta.group
-
+                
                 if meta.group not in group_map["groups"]:
                     group_map["groups"][meta.group] = {"current": None, "fields": set()}
                 group_map["groups"][meta.group]["fields"].add(field)
-
+            
             if getattr(self, field.name) != PLACEHOLDER:
                 # Skip anything not set to the sentinel value
                 all_sentinel = False
-
+                
                 if meta.group:
                     # This was set, so make it the selected value of the one-of.
                     group_map["groups"][meta.group]["current"] = field
-
+                
                 continue
-
+            
             setattr(self, field.name, self._get_field_default(field, meta))
-
+        
         # Now that all the defaults are set, reset it!
         self.__dict__["_serialized_on_wire"] = not all_sentinel
         self.__dict__["_unknown_fields"] = b""
         self.__dict__["_group_map"] = group_map
-
+    
     def __setattr__(self, attr: str, value: Any) -> None:
         if attr != "_serialized_on_wire":
             # Track when a field has been set.
             self.__dict__["_serialized_on_wire"] = True
-
+        
         if attr in getattr(self, "_group_map", {}).get("fields", {}):
             group = self._group_map["fields"][attr]
             for field in self._group_map["groups"][group]["fields"]:
@@ -486,9 +487,9 @@ class Message(ABC):
                         field.name,
                         self._get_field_default(field, FieldMetadata.get(field)),
                     )
-
+        
         super().__setattr__(attr, value)
-
+    
     def __bytes__(self) -> bytes:
         """
         Get the binary encoded Protobuf representation of this instance.
@@ -497,25 +498,25 @@ class Message(ABC):
         for field in dataclasses.fields(self):
             meta = FieldMetadata.get(field)
             value = getattr(self, field.name)
-
+            
             if value is None:
                 # Optional items should be skipped. This is used for the Google
                 # wrapper types.
                 continue
-
+            
             # Being selected in a a group means this field is the one that is
             # currently set in a `oneof` group, so it must be serialized even
             # if the value is the default zero value.
             selected_in_group = False
             if meta.group and self._group_map["groups"][meta.group]["current"] == field:
                 selected_in_group = True
-
+            
             serialize_empty = False
             if isinstance(value, Message) and value._serialized_on_wire:
                 # Empty messages can still be sent on the wire if they were
                 # set (or received empty).
                 serialize_empty = True
-
+            
             if value == self._get_field_default(field, meta) and not (
                     selected_in_group or serialize_empty
             ):
@@ -524,7 +525,7 @@ class Message(ABC):
                 # serialize an empty message (i.e. zero value was explicitly
                 # set by the user).
                 continue
-
+            
             if isinstance(value, list):
                 if meta.proto_type in PACKED_TYPES:
                     # Packed lists look like a length-delimited field. First,
@@ -553,27 +554,27 @@ class Message(ABC):
                     serialize_empty=serialize_empty,
                     wraps=meta.wraps or "",
                 )
-
+        
         return output + self._unknown_fields
-
+    
     # For compatibility with other libraries
     SerializeToString = __bytes__
-
+    
     def _type_hint(self, field_name: str) -> Type:
         module = inspect.getmodule(self.__class__)
         type_hints = get_type_hints(self.__class__, vars(module))
         return type_hints[field_name]
-
+    
     def _cls_for(self, field: dataclasses.Field, index: int = 0) -> Type:
         """Get the message class for a field from the type hints."""
         cls = self._type_hint(field.name)
         if hasattr(cls, "__args__") and index >= 0:
             cls = cls.__args__[index]
         return cls
-
+    
     def _get_field_default(self, field: dataclasses.Field, meta: FieldMetadata) -> Any:
         t = self._type_hint(field.name)
-
+        
         value: Any = 0
         if hasattr(t, "__origin__"):
             if t.__origin__ in (dict, Dict):
@@ -598,9 +599,9 @@ class Message(ABC):
             # This is either a primitive scalar or another message type. Calling
             # it should result in its zero value.
             value = t()
-
+        
         return value
-
+    
     def _postprocess_single(
             self, wire_type: int, meta: FieldMetadata, field: dataclasses.Field, value: Any
     ) -> Any:
@@ -625,7 +626,7 @@ class Message(ABC):
                 value = value.decode("utf-8")
             elif meta.proto_type == TYPE_MESSAGE:
                 cls = self._cls_for(field)
-
+                
                 if cls == datetime:
                     value = _Timestamp().parse(value).to_datetime()
                 elif cls == timedelta:
@@ -652,9 +653,9 @@ class Message(ABC):
                     bases=(Message,),
                 )
                 value = Entry().parse(value)
-
+        
         return value
-
+    
     def parse(self: T, data: bytes) -> T:
         """
         Parse the binary encoded Protobuf into this message instance. This
@@ -665,7 +666,7 @@ class Message(ABC):
             if parsed.number in fields:
                 field = fields[parsed.number]
                 meta = FieldMetadata.get(field)
-
+                
                 value: Any
                 if (
                         parsed.wire_type == WIRE_LEN_DELIM
@@ -692,7 +693,7 @@ class Message(ABC):
                     value = self._postprocess_single(
                         parsed.wire_type, meta, field, parsed.value
                     )
-
+                
                 current = getattr(self, field.name)
                 if meta.proto_type == TYPE_MAP:
                     # Value represents a single key/value pair entry in the map.
@@ -703,14 +704,14 @@ class Message(ABC):
                     setattr(self, field.name, value)
             else:
                 self._unknown_fields += parsed.raw
-
+        
         return self
-
+    
     # For compatibility with other libraries.
     @classmethod
     def FromString(cls: Type[T], data: bytes) -> T:
         return cls().parse(data)
-
+    
     def to_dict(
             self, casing: Casing = Casing.CAMEL, include_default_values: bool = False
     ) -> dict:
@@ -751,7 +752,7 @@ class Message(ABC):
                 for k in v:
                     if hasattr(v[k], "to_dict"):
                         v[k] = v[k].to_dict(casing, include_default_values)
-
+                
                 if v or include_default_values:
                     output[cased_name] = v
             elif v != self._get_field_default(field, meta) or include_default_values:
@@ -774,7 +775,7 @@ class Message(ABC):
                 else:
                     output[cased_name] = v
         return output
-
+    
     def from_dict(self: T, value: dict) -> T:
         """
         Parse the key/value pairs in `value` into this message instance. This
@@ -787,7 +788,7 @@ class Message(ABC):
             if snake_cased in fields_by_name:
                 field = fields_by_name[snake_cased]
                 meta = FieldMetadata.get(field)
-
+                
                 if value[key] is not None:
                     if meta.proto_type == "message":
                         v = getattr(self, field.name)
@@ -830,21 +831,53 @@ class Message(ABC):
                                 v = [enum_cls.from_string(e) for e in v]
                             elif isinstance(v, str):
                                 v = enum_cls.from_string(v)
-
+                        
                         if v is not None:
                             setattr(self, field.name, v)
         return self
-
+    
     def to_json(self, indent: Union[None, int, str] = None) -> str:
         """Returns the encoded JSON representation of this message instance."""
         return json.dumps(self.to_dict(), indent=indent)
-
+    
     def from_json(self: T, value: Union[str, bytes]) -> T:
         """
         Parse the key/value pairs in `value` into this message instance. This
         returns the instance itself and is therefore assignable and chainable.
         """
         return self.from_dict(json.loads(value))
+    
+    def check_ok(self, http_code=200):
+        """
+        check response http status code
+        {self.status_code}
+        :param http_code:
+        :return:
+        """
+        assert self.status_code == http_code, f'[校验失败,http_code 实际值:{self.status_code} == 预期值：{http_code}]'
+        logger.info(f'[校验成功,http_code 实际值:{self.status_code} == 预期值：{http_code}]')
+        return self
+    
+    def check_code(self, code=0):
+        """
+        check response body code
+        {self.rep.code}
+        :param code: response code number
+        :return:
+        """
+        assert self.code == code, f'[校验失败,response.code 实际值:{self.code} == 预期值：{code}]'
+        logger.info(f'[校验成功,response.code 实际值:{self.code} == 预期值：{code}]')
+        return self
+    
+    def check_values(self, expect_val, actual_val):
+        """
+        check response data
+        @expect_val : 预期值
+        @actual_val : 实际值,取自response data
+        """
+        assert actual_val == expect_val, f'[校验失败,实际值:{actual_val} == 预期值：{expect_val}]'
+        logger.info(f'[校验成功,实际值:{actual_val} == 预期值：{expect_val}]')
+        return self
 
 
 def serialized_on_wire(message: Message) -> bool:
@@ -860,7 +893,7 @@ def which_one_of(message: Message, group_name: str) -> Tuple[str, Any]:
     """Return the name and value of a message's one-of field group."""
     field = message._group_map["groups"].get(group_name, {}).get("current")
     if not field:
-        return ("", None)
+        return "", None
     return (field.name, getattr(message, field.name))
 
 
@@ -876,10 +909,10 @@ class _Duration(Message):
     # a non-zero value for the `nanos` field must be of the same sign as the
     # `seconds` field. Must be from -999,999,999 to +999,999,999 inclusive.
     nanos: int = int32_field(2)
-
+    
     def to_timedelta(self) -> timedelta:
         return timedelta(seconds=self.seconds, microseconds=self.nanos / 1e3)
-
+    
     @staticmethod
     def delta_to_json(delta: timedelta) -> str:
         parts = str(delta.total_seconds()).split(".")
@@ -898,11 +931,11 @@ class _Timestamp(Message):
     # second values with fractions must still have non-negative nanos values that
     # count forward in time. Must be from 0 to 999,999,999 inclusive.
     nanos: int = int32_field(2)
-
+    
     def to_datetime(self) -> datetime:
         ts = self.seconds + (self.nanos / 1e9)
         return datetime.fromtimestamp(ts, tz=timezone.utc)
-
+    
     @staticmethod
     def timestamp_to_json(dt: datetime) -> str:
         nanos = dt.microsecond * 1e3
@@ -927,12 +960,12 @@ class _WrappedMessage(Message):
     Google protobuf wrapper types base class. JSON representation is just the
     value itself.
     """
-
+    
     value: Any
-
+    
     def to_dict(self, casing: Casing = Casing.CAMEL) -> Any:
         return self.value
-
+    
     def from_dict(self: T, value: Any) -> T:
         if value is not None:
             self.value = value
@@ -1006,7 +1039,7 @@ class ServiceStub(ABC):
     session.host = None
     session.suffix_url = None
     session.common_args = None
-
+    
     def __init__(self, target: str = None, common_args: Dict[str, str] = {}):
         self._common_args = common_args
         if session.suffix_url is None:
@@ -1015,13 +1048,11 @@ class ServiceStub(ABC):
         if session.host is None:
             session.host = target
         self._target = session.host
-
+        
         if session.common_args is None:
             session.common_args = common_args
-
+    
     def _do_twirp_rpc(self, api, request: Message, response: Message):
-        # if len(self._common_args) > 0:
-        #     api = api + "?" + urlencode(self._common_args)
         api = api + self.suffix_url
         url = self._target + "/twirp" + api
         logger.info(f'[请求URL]:{url}')
@@ -1036,11 +1067,7 @@ class ServiceStub(ABC):
         logger.info(f'[请求结果]:{r.json()}')
         logger.info(f'[请求http_code]:{r.status_code}')
         logger.info(f'[请求耗时]:{r.elapsed.total_seconds()}秒')
-        # try:
-        #     resp = req.
-        # except HTTPError as err:
-        #     raise TwirpException.from_http_err(err)
-
+        response.__dict__.update(r.__dict__)
         response.from_json(r.content)
 
 
@@ -1050,7 +1077,7 @@ class TwirpException(httplib.HTTPException):
         self.message = message
         self.meta = meta
         super(TwirpException, self).__init__(message)
-
+    
     @classmethod
     def from_http_err(cls, err):
         try:
